import os, sys, stat, inspect, traceback

from htmlhelpgen import *

INPUT_DIRECTORY = "C:\\Python22\\lib"
OUTPUT_DIRECTORY = "C:\\Python22\\libdocs"
INCLUDE_SOURCE = 0

# sample code: generate html help tree --------------------------------------

def extension(x):
    pos = x.rfind(".")
    if pos >= 0:
        return x[pos:].lower()

def basename(x):
    pos = x.rfind(".")
    if pos >= 0:
        return x[:pos]
    return x

file_index = 0

def GetItem(members,name):
    for member in members:
        if member[0] == name:
            return member

def EscapeHtmlString(s):
    if not s:
        return ""
    for k in ( ("<","&lt;"),
               (">","&gt;") ):
        s = s.replace(k[0],k[1])
    return "<pre>"+s+"</pre>"

def PrintFunctions(file,members,node,module_name):
    global file_index
    title_printed = 0
    for item in members:
        if not inspect.isfunction(item[1]):
            continue
        
        if not title_printed:
            print >>file, "<h2>Functions</h2><ul>"
            title_printed = 1

        topic = item[0]
        doc, comment = inspect.getdoc(item[1]), inspect.getcomments(item[1])
        if not doc and not comment:
            print >>file, '<li>%s</li>' % (topic)
            continue

        html_filename = "data%05d.htm" % file_index
        html_pathname = os.path.join(OUTPUT_DIRECTORY,html_filename)
        file_index += 1
        subject = HTMLHelpSubject(topic, html_filename)
        node.subjects.append( subject )
        subject.keywords[0] = "%s() (in module %s)" % (topic, module_name)
        html_file = open(html_pathname,"w")
        print >>html_file, """<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2 Final//EN"><HTML><head>
<META HTTP-EQUIV="Content-Type" Content="text/html; charset=Windows-1252">
<meta http-equiv="expires" content="0">
<meta http-equiv="pragma" content="no-cache">
<title>%s</title>
<link rel="stylesheet" type="text/css" href="pydoc.css"></HEAD>
<body link="#000080" vlink="#000080" bgcolor="#FFFFFF" text="#000000">
<h1>%s</h1><pre>""" % (topic, topic)
        if doc:
            print >>html_file, EscapeHtmlString(doc)
            if comment:
                print >>html_file, "<h2>Comments</h2>"
                print >>html_file, EscapeHtmlString(comment)
        else:
            print >>html_file, EscapeHtmlString(comment)

        print >>html_file, "</pre>"
        print >>html_file, "</body></html>"
        html_file.close()
        print >>file, '<li><a href="%s">%s</a></li>' % (html_filename, topic)
    if title_printed:
        print >>file, "</ul>"

def PrintClasses(file,members,node,module_name):
    global file_index
    title_printed = 0
    for item in members:
        if not inspect.isclass(item[1]):
            continue
        
        if not title_printed:
            print >>file, "<h2>Classes</h2><ul>"
            title_printed = 1

        object = item[1]
        topic = item[0]
        
        doc, comment, members = inspect.getdoc(object), inspect.getcomments(object), inspect.getmembers(object)
        if not (doc or comment or members):
            print >>file, '<li>%s</li>' % (topic)
            continue

        html_filename = "data%05d.htm" % file_index
        html_pathname = os.path.join(OUTPUT_DIRECTORY,html_filename)       
        file_index += 1
        
        subject = HTMLHelpSubject(topic, html_filename)
        node.subjects.append( subject )
        subject.keywords[0] = "class %s (in module %s)" % (topic, module_name)
        html_file = open(html_pathname,"w")

        print >>html_file, """<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2 Final//EN"><HTML><head>
<META HTTP-EQUIV="Content-Type" Content="text/html; charset=Windows-1252">
<meta http-equiv="expires" content="0">
<meta http-equiv="pragma" content="no-cache">
<title>%s</title>
<link rel="stylesheet" type="text/css" href="pydoc.css"></HEAD>
<body link="#000080" vlink="#000080" bgcolor="#FFFFFF" text="#000000">
<h1>%s</h1>""" % (topic, topic)
        if doc:
            print >>html_file, EscapeHtmlString(doc)
            if comment:
                print >>html_file, "<h2>Comments</h2>"
                print >>html_file, EscapeHtmlString(comment)
        else:
            print >>html_file, EscapeHtmlString(comment)

        if members:
            print >>html_file, "<h2>Members</h2>"
            for member in members:
                memberfunc = getattr(object, member[0])

                doc, comment =   inspect.getdoc(memberfunc), inspect.getcomments(memberfunc)
                s = ""
                if not doc:
                    if comment:
                        s = comment
                else:
                    s = doc
                    if comment:
                        s = s + "<br>\n" + comment

                if s:
                    sub_html_filename = "data%05d.htm" % file_index
                    sub_html_pathname = os.path.join(OUTPUT_DIRECTORY,sub_html_filename)       
                    file_index += 1
                    
                    sub_subject = HTMLHelpSubject(member[0], sub_html_filename)
                    subject.subjects.append( sub_subject )
                    sub_subject.keywords[0] = "%s.%s() (in module %s)" % (topic, member[0], module_name)
                    sub_html_file = open(sub_html_pathname,"w")
                    print >>sub_html_file, """<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2 Final//EN"><HTML><head>
<META HTTP-EQUIV="Content-Type" Content="text/html; charset=Windows-1252">
<meta http-equiv="expires" content="0">
<meta http-equiv="pragma" content="no-cache">
<title>%s</title>
<link rel="stylesheet" type="text/css" href="pydoc.css"></HEAD>
<body link="#000080" vlink="#000080" bgcolor="#FFFFFF" text="#000000">
<h1>%s.%s</h1>""" % (member[0], topic, member[0])

                    print >>sub_html_file, EscapeHtmlString(s)
                    print >>sub_html_file, "</body></html>"

                    sub_html_file.close()                    
                    print >>html_file, '<p><a href="%s">%s.%s</a></p>' % (sub_html_filename, topic, member[0] )
                else:
                    print >>html_file, '<p>%s</p>' % (member[0])

        html_file.close()
        print >>file, '<li><a href="%s">%s</a></li>' % (html_filename, topic)
    if title_printed:
        print >>file, "</ul>"

def GenerateHtmlDocFile( file, module, node, pathname ):
    global file_index
    print >>file, """<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2 Final//EN"><HTML><head>
<META HTTP-EQUIV="Content-Type" Content="text/html; charset=Windows-1252">
<meta http-equiv="expires" content="0">
<meta http-equiv="pragma" content="no-cache">
<title>%s</title>
<link rel="stylesheet" type="text/css" href="pydoc.css"></HEAD>
<body link="#000080" vlink="#000080" bgcolor="#FFFFFF" text="#000000">
<h1>module %s</h1>""" % (module, module)

    try:
        exec("import %s" % module)
        members = eval("inspect.getmembers(%s)" % module)

        doc = GetItem(members,"__doc__")
        if doc:
            print >>file, EscapeHtmlString(doc[1])

        PrintClasses(file,members,node,module)
        PrintFunctions(file,members,node,module)

        # finally, generate raw file data
        filename = GetItem(members,"__file__")
        if filename and INCLUDE_SOURCE:
            filename = filename[1]
            if extension(filename) == ".pyc":
                filename = basename(filename) + ".py"

            html_filename = "data%05d.htm" % file_index
            html_pathname = os.path.join(OUTPUT_DIRECTORY,html_filename)       
            file_index += 1
            topic = filename
            node.subjects.append( HTMLHelpSubject(topic, html_filename) )
            html_file = open(html_pathname,"w")
            html_file.write("<html><body><pre>\n")
            html_file.write(EscapeHtmlString(open(filename).read()))
            html_file.write("\n</pre></body></html>)\n")
            html_file.close()
            print >>file, '<h2>Filename</h2><p><a href="%s">%s</a></p>' % (html_filename, filename)
            #print >>file, '<h2>Filename</h2><p>%s</p>' % (filename)
            
        exec("del %s" % module)        
    except:
        traceback.print_exception(sys.exc_type, sys.exc_value, sys.exc_traceback, 5, file)
        print >>file, "<p>ERROR, unable to inspect %s.</p>" % pathname

        #try:
        #    file.write("<pre>\n")
        #    file.write(EscapeHtmlString(open(pathname).read()))
        #    file.write("\n</pre>\n")            
        #except:
        #    pass
    
    print >>file, "</body></html>"

def walktree(dir, importpath, node, root_file):
    global file_index
    print "WALKTREE", importpath

    print >>root_file, """<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2 Final//EN"><HTML><head>
<META HTTP-EQUIV="Content-Type" Content="text/html; charset=Windows-1252">
<meta http-equiv="expires" content="0">
<meta http-equiv="pragma" content="no-cache">
<link rel="stylesheet" type="text/css" href="pydoc.css"></HEAD>
<body link="#000080" vlink="#000080" bgcolor="#FFFFFF" text="#000000"><ul>"""
   
    for f in os.listdir(dir):
        
        pathname = os.path.join(dir, f)
        
        mode = os.stat(pathname)[stat.ST_MODE]       

        topic = ".".join(importpath + [basename(f)])
        html_filename = "data%05d.htm" % file_index
        html_pathname = os.path.join(OUTPUT_DIRECTORY,html_filename)       

        if stat.S_ISDIR(mode):
            # hardcoded bugfix: do not try to import the "test" modules...
            if f == "test":
                continue
            file_index += 1
            html_file = open(html_pathname,"w")        
            print >>root_file, '<li><a href="%s">%s</a></li>' % (html_filename, topic)

            node.subjects.append( HTMLHelpSubject(topic, html_filename) )

            # yet another hardcoded bugfix:            
            if f == "site-packages":
                walktree( pathname, importpath, node.subjects[-1], html_file )
            else:
                walktree( pathname, importpath + [f], node.subjects[-1], html_file )
        elif stat.S_ISREG(mode) and extension(f) == '.py':
            file_index += 1
            html_file = open(html_pathname,"w")        
            print >>root_file, '<li><a href="%s">%s</a></li>' % (html_filename, topic)

            node.subjects.append( HTMLHelpSubject(topic, html_filename) )            
            GenerateHtmlDocFile( html_file, topic, node.subjects[-1], pathname )
        else:
            continue

        html_file.close()
    print >>root_file, "</ul></body></html>"

project = HTMLHelpProject( "Python 2.2", "default.htm" )

root_file = open(os.path.join(OUTPUT_DIRECTORY,"default.htm"),"w")
walktree( INPUT_DIRECTORY, [], project.root, root_file )
root_file.close()

project.Generate(OUTPUT_DIRECTORY)

